#include "ep_xfig.h"

#include "printer.h"
#include "wallideal.h"
#include "lp.h"


void XfigEnumerationPrinter::Point::print()const
{
  fprintf(Stderr,"(%f,%f,%f)\n",x,y,z);
}

XfigEnumerationPrinter::XfigEnumerationPrinter(bool _largerTriangle):
  largerTriangle(_largerTriangle),
  variableShift(0)
{
}


void XfigEnumerationPrinter::onOpened()
{
  fprintf(file,"#FIG 3.2\n"
          "Landscape\n"
          "Center\n"
          "Inches\n"
          "Letter\n"
          "100.00\n"
          "Single\n"
          "-2\n"
          "1200 2\n");
}


void XfigEnumerationPrinter::onClose()
{
}


void XfigEnumerationPrinter::onClosed()
{
}


void XfigEnumerationPrinter::beginEnumeration(const PolynomialSet &groebnerBasis)
{
  basisCounter=0;
}


void XfigEnumerationPrinter::endEnumeration()
{
}


void XfigEnumerationPrinter::printPolygon(const Polygon &p)
{
  fprintf(Stderr,"{\n");
  for(Polygon::const_iterator i=p.begin();i!=p.end();i++)
    i->print();
  fprintf(Stderr,"}\n");
}


XfigEnumerationPrinter::Polygon XfigEnumerationPrinter::intersect(const Polygon &polygon, const Point &normal)
{
  //fprintf(Stderr,"intersect input:\n");
  //printPolygon(polygon);

  Polygon ret;
  Polygon temp;

  Polygon::const_iterator i=polygon.begin();
  while(i!=polygon.end())
    {
      if(normal.isInside(*i))break;
      i++;
    }

  if(i==polygon.end())return ret;
  
  for(Polygon::const_iterator j=i;j!=polygon.end();j++)
    temp.push_back(*j);
  for(Polygon::const_iterator j=polygon.begin();j!=i;j++)
    temp.push_back(*j);
  temp.push_back(*i);

//  fprintf(Stderr,"intersect temp:\n");
//  printPolygon(temp);

  Polygon::const_iterator j=temp.begin();


  while(j!=temp.end())
    {
      Point last(0,0,0);
      while(j!=temp.end()&&normal.isInside(*j))
        {
          //fprintf(Stderr,"inside:");
          //j->print();
          last=*j;
          ret.push_back(*j);
          j++;
        }
      if(j==temp.end())break;
      Point leaving=normal.intersect(last,*j);
      while(!normal.isInside(*j))
        {
          //fprintf(Stderr,"outside:");
          //j->print();
          last=*j;
          j++;
        }        
      Point entering=normal.intersect(last,*j);
      ret.push_back(leaving);
      ret.push_back(entering);
      ret.push_back(*j);
      j++;
    }
  ret.pop_back();

  //  fprintf(Stderr,"intersect ret:\n");
  //  printPolygon(ret);

  return ret;
}


void XfigEnumerationPrinter::kickPoint(const Point &p)
{
  //  fprintf(file," %i %i",(int)(p.x*10000),(int)(p.y*10000));
  float invSqrt2=0.707106781;
  float invSqrt6=0.40824829;
  fprintf(file," %i %i",(int)((invSqrt2+invSqrt2*p.x-invSqrt2*p.y)*10000),(int)((2*invSqrt6+invSqrt6*p.x+invSqrt6*p.y-2*invSqrt6*p.z)*10000));
}


void XfigEnumerationPrinter::drawPolygon(const Polygon &vertices)
{
  if(vertices.size())
    {
      fprintf(file,"2 3 0 1 0 %i 50 0 25 0.000 0 0 -1 0 0 %i\n        ",7,vertices.size()+1);
      
      for(Polygon::const_iterator i=vertices.begin();i!=vertices.end();i++)
        kickPoint(*i);
      kickPoint(*vertices.begin());
      fprintf(file,"\n");
    }
  fflush(file);
}


bool XfigEnumerationPrinter::basis(const PolynomialSet &groebnerBasis)
{
  list<Point> p;

  if(largerTriangle)
    {
      p.push_back(Point(3,-1,-1));
      p.push_back(Point(-1,3,-1));
      p.push_back(Point(-1,-1,3));
    }
  else
    {
      p.push_back(Point(1,0,0));
      p.push_back(Point(0,1,0));
      p.push_back(Point(0,0,1));
    }

  IntegerVectorList normals=wallInequalities(groebnerBasis);
  for(IntegerVectorList::const_iterator i=normals.begin();i!=normals.end();i++)
    //if(wallContainsPositiveVector(*i))
    if(isFacet(normals,i))
        {
          if(i->size()>=3)
            {
              AsciiPrinter(Stderr).printVector(*i);
	      Point n((*i)[variableShift%i->size()],(*i)[(variableShift+1)%i->size()],(*i)[(variableShift+2)%i->size()]);
              p=intersect(p,n);
	      fprintf(Stderr,"%i\n",variableShift);
            }
        }

  printPolygon(p);
  drawPolygon(p);

  basisCounter++;

  fprintf(Stderr,"basisCounter:%i\n",basisCounter);

  return true;
}


string XfigEnumerationPrinter::extension()
{
  return "";
}

void XfigEnumerationPrinter::setVariableShift(int shift)
{
  variableShift=shift;
}
